@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2020 The Fluent Bit Authors
+ *  Copyright (C) 2019-2021 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -196,10 +196,10 @@ struct flb_out_datadog *flb_datadog_conf_create(struct flb_output_instance *ins,
                                        ctx->proxy_host,
                                        ctx->proxy_port,
                                        io_flags,
-                                       &ins->tls);
+                                       ins->tls);
     }
     else {
-        upstream = flb_upstream_create(config, ctx->host, ctx->port, io_flags, &ins->tls);
+        upstream = flb_upstream_create(config, ctx->host, ctx->port, io_flags, ins->tls);
     }
 
     if (!upstream) {
@@ -208,6 +208,7 @@ struct flb_out_datadog *flb_datadog_conf_create(struct flb_output_instance *ins,
         return NULL;
     }
     ctx->upstream = upstream;
+    flb_output_upstream_set(ctx->upstream, ins);
 
     return ctx;
 }
