@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2020 The Fluent Bit Authors
+ *  Copyright (C) 2019-2021 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -107,7 +107,11 @@ int produce_message(struct flb_time *tm, msgpack_object *map,
     msgpack_object val;
     flb_sds_t s;
 
-    /* Init temporary buffers */
+    flb_debug("in produce_message\n");
+    if (flb_log_check(FLB_LOG_DEBUG))
+        msgpack_object_print(stderr, *map);
+
+    /* Init temporal buffers */
     msgpack_sbuffer_init(&mp_sbuf);
     msgpack_packer_init(&mp_pck, &mp_sbuf, msgpack_sbuffer_write);
 
@@ -250,6 +254,23 @@ int produce_message(struct flb_time *tm, msgpack_object *map,
         out_buf = s;
         out_size = flb_sds_len(s);
     }
+#ifdef FLB_HAVE_AVRO_ENCODER
+    else if (ctx->format == FLB_KAFKA_FMT_AVRO) {
+        flb_plg_debug(ctx->ins, "calling flb_msgpack_raw_to_avro_sds\n");
+        flb_plg_debug(ctx->ins, "avro schema ID:%s:\n", ctx->avro_fields.schema_id);
+        flb_plg_debug(ctx->ins, "avro schema string:%s:\n", ctx->avro_fields.schema_str);
+        s = flb_msgpack_raw_to_avro_sds(mp_sbuf.data, mp_sbuf.size, &ctx->avro_fields);
+        if(!s) {
+            flb_plg_error(ctx->ins, "error encoding to AVRO:schema:%s:schemaID:%s:\n", ctx->avro_fields.schema_str, ctx->avro_fields.schema_id);
+            msgpack_sbuffer_destroy(&mp_sbuf);
+            return FLB_ERROR;
+        }
+        out_buf = s;
+        out_size = flb_sds_len(s);
+        // schema_id is binary. skip it: schema_id + avro packaging
+        flb_debug("back from flb_msgpack_raw_to_avro_sds:out_size:%zu:val:%s:\n", out_size, out_buf+strlen(ctx->avro_fields.schema_id)+3);
+    }
+#endif
 
     if (!message_key) {
         message_key = ctx->message_key;
@@ -278,6 +299,11 @@ int produce_message(struct flb_time *tm, msgpack_object *map,
             flb_sds_destroy(s);
         }
         msgpack_sbuffer_destroy(&mp_sbuf);
+        /*
+         * Unblock the flush requests so that the
+         * engine could try sending data again.
+         */
+        ctx->blocked = FLB_FALSE;
         return FLB_RETRY;
     }
 
@@ -287,8 +313,9 @@ int produce_message(struct flb_time *tm, msgpack_object *map,
                            out_buf, out_size,
                            message_key, message_key_len,
                            ctx);
+
     if (ret == -1) {
-        fprintf(stderr,
+        flb_error(
                 "%% Failed to produce to topic %s: %s\n",
                 rd_kafka_topic_name(topic->tp),
                 rd_kafka_err2str(rd_kafka_last_error()));
@@ -317,7 +344,7 @@ int produce_message(struct flb_time *tm, msgpack_object *map,
              * to enqueue this message, if we exceed 10 times, we just
              * issue a full retry of the data chunk.
              */
-            flb_time_sleep(1000, config);
+            flb_time_sleep(1000);
             rd_kafka_poll(ctx->producer, 0);
 
             /* Issue a re-try */
@@ -338,6 +365,11 @@ int produce_message(struct flb_time *tm, msgpack_object *map,
     if (ctx->format == FLB_KAFKA_FMT_GELF) {
         flb_sds_destroy(s);
     }
+#ifdef FLB_HAVE_AVRO_ENCODER
+    if (ctx->format == FLB_KAFKA_FMT_AVRO) {
+        flb_sds_destroy(s);
+    }
+#endif
 
     msgpack_sbuffer_destroy(&mp_sbuf);
     return FLB_OK;
