@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2020 The Fluent Bit Authors
+ *  Copyright (C) 2019-2021 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -139,35 +139,35 @@ static int cb_firehose_init(struct flb_output_instance *ins,
     }
 
     /* one tls instance for provider, one for cw client */
-    ctx->cred_tls.context = flb_tls_context_new(FLB_TRUE,
-                                                ins->tls_debug,
-                                                ins->tls_vhost,
-                                                ins->tls_ca_path,
-                                                ins->tls_ca_file,
-                                                ins->tls_crt_file,
-                                                ins->tls_key_file,
-                                                ins->tls_key_passwd);
+    ctx->cred_tls = flb_tls_create(FLB_TRUE,
+                                   ins->tls_debug,
+                                   ins->tls_vhost,
+                                   ins->tls_ca_path,
+                                   ins->tls_ca_file,
+                                   ins->tls_crt_file,
+                                   ins->tls_key_file,
+                                   ins->tls_key_passwd);
 
-    if (!ctx->cred_tls.context) {
+    if (!ctx->cred_tls) {
         flb_plg_error(ctx->ins, "Failed to create tls context");
         goto error;
     }
 
-    ctx->client_tls.context = flb_tls_context_new(FLB_TRUE,
-                                                  ins->tls_debug,
-                                                  ins->tls_vhost,
-                                                  ins->tls_ca_path,
-                                                  ins->tls_ca_file,
-                                                  ins->tls_crt_file,
-                                                  ins->tls_key_file,
-                                                  ins->tls_key_passwd);
-    if (!ctx->client_tls.context) {
+    ctx->client_tls = flb_tls_create(FLB_TRUE,
+                                     ins->tls_debug,
+                                     ins->tls_vhost,
+                                     ins->tls_ca_path,
+                                     ins->tls_ca_file,
+                                     ins->tls_crt_file,
+                                     ins->tls_key_file,
+                                     ins->tls_key_passwd);
+    if (!ctx->client_tls) {
         flb_plg_error(ctx->ins, "Failed to create tls context");
         goto error;
     }
 
     ctx->aws_provider = flb_standard_chain_provider_create(config,
-                                                           &ctx->cred_tls,
+                                                           ctx->cred_tls,
                                                            (char *) ctx->region,
                                                            ctx->sts_endpoint,
                                                            NULL,
@@ -187,15 +187,15 @@ static int cb_firehose_init(struct flb_output_instance *ins,
         }
 
         /* STS provider needs yet another separate TLS instance */
-        ctx->sts_tls.context = flb_tls_context_new(FLB_TRUE,
-                                                   ins->tls_debug,
-                                                   ins->tls_vhost,
-                                                   ins->tls_ca_path,
-                                                   ins->tls_ca_file,
-                                                   ins->tls_crt_file,
-                                                   ins->tls_key_file,
-                                                   ins->tls_key_passwd);
-        if (!ctx->sts_tls.context) {
+        ctx->sts_tls = flb_tls_create(FLB_TRUE,
+                                      ins->tls_debug,
+                                      ins->tls_vhost,
+                                      ins->tls_ca_path,
+                                      ins->tls_ca_file,
+                                      ins->tls_crt_file,
+                                      ins->tls_key_file,
+                                      ins->tls_key_passwd);
+        if (!ctx->sts_tls) {
             flb_errno();
             goto error;
         }
@@ -203,7 +203,7 @@ static int cb_firehose_init(struct flb_output_instance *ins,
         ctx->base_aws_provider = ctx->aws_provider;
 
         ctx->aws_provider = flb_sts_provider_create(config,
-                                                    &ctx->sts_tls,
+                                                    ctx->sts_tls,
                                                     ctx->base_aws_provider,
                                                     NULL,
                                                     (char *) ctx->role_arn,
@@ -225,6 +225,7 @@ static int cb_firehose_init(struct flb_output_instance *ins,
     /* initialize credentials and set to sync mode */
     ctx->aws_provider->provider_vtable->sync(ctx->aws_provider);
     ctx->aws_provider->provider_vtable->init(ctx->aws_provider);
+    ctx->aws_provider->provider_vtable->upstream_set(ctx->aws_provider, ctx->ins);
 
     if (ctx->endpoint == NULL) {
         ctx->endpoint = flb_aws_endpoint("firehose", (char *) ctx->region);
@@ -251,7 +252,7 @@ static int cb_firehose_init(struct flb_output_instance *ins,
 
     struct flb_upstream *upstream = flb_upstream_create(config, ctx->endpoint,
                                                         443, FLB_IO_TLS,
-                                                        &ctx->client_tls);
+                                                        ctx->client_tls);
     if (!upstream) {
         flb_plg_error(ctx->ins, "Connection initialization error");
         goto error;
@@ -347,16 +348,16 @@ void flb_firehose_ctx_destroy(struct flb_firehose *ctx)
             flb_aws_provider_destroy(ctx->aws_provider);
         }
 
-        if (ctx->cred_tls.context) {
-            flb_tls_context_destroy(ctx->cred_tls.context);
+        if (ctx->cred_tls) {
+            flb_tls_destroy(ctx->cred_tls);
         }
 
-        if (ctx->sts_tls.context) {
-            flb_tls_context_destroy(ctx->sts_tls.context);
+        if (ctx->sts_tls) {
+            flb_tls_destroy(ctx->sts_tls);
         }
 
-        if (ctx->client_tls.context) {
-            flb_tls_context_destroy(ctx->client_tls.context);
+        if (ctx->client_tls) {
+            flb_tls_destroy(ctx->client_tls);
         }
 
         if (ctx->firehose_client) {
