@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2020 The Fluent Bit Authors
+ *  Copyright (C) 2019-2021 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -31,6 +31,9 @@
 #include <fluent-bit/flb_sds.h>
 
 struct flb_splunk {
+    /* Payload compression */
+    int compress_gzip;
+
     /* HTTP Auth */
     char *http_user;
     char *http_passwd;
